rm(list = ls())
library(dfoptim)
library(ape)
library(phylolm)
library(MASS)

setwd("~/DID/BEEHIVE/GWAS/RGWAS/to_share/")
load("heritability_subtypeB_data.RData")

source("all.functions_2.R") # various functions for inference
source("transf.branch.lengths.R")

########################################       EXAMPLE INFERENCE    ######################################

# 1. FIT NULL MODEL
lm0 <- lm(BEEHIVE_LVL ~ GENDER + t.mode + age.cat + ethnic2 + assay.gsvl2, data = VL.info)
res.variance <- summary(lm0)$sigma^2 # residual variance of the linear model
opt0 <- list(lik = as.numeric(- logLik(lm0)), pars = res.variance)

mm <- model.matrix(lm0) # model matrix for this linear model
if(ncol(mm) > 1){
  mm <- mm[, 2:ncol(mm), drop = F]
} else {
  mm <- NULL
}

# 2. FIT BM AND OU MODELS

my.init.fun.BM <- function() return(runif(2, c(0.01, 0.01), c(1, 1))) # function drawing starting point of inference
my.init.fun.OU <- function() return(runif(3, c(0.01, 0.01, 0.01), c(1, 1, 0.1)))
my.lower.BM <- c(10^-6, 10^-6)
my.upper.BM <- c(10, 10)
my.lower.OU <- c(10^-6, 10^-6, 10^-6)
my.upper.OU <- c(10, 10, 10)

VL <- VL.info$BEEHIVE_LVL # vector of phenotype
names(VL) <- VL.info$ID
n <- length(VL)

treeBM <- reorder.phylo(subtree, "pruningwise") # reordering tree for BM
treeOU <- reorder.phylo(subtree, "pruningwise") # reordering and tree manipulation for OU
D <- numeric(n)
des <- treeOU$edge[, 2]
externalEdge <- (des <= n)
root.to.tip <- pruningwise.distFromRoot(treeOU)[1:n]
if (!is.ultrametric(treeOU)) {
  flag <- 1
  dis <- pruningwise.distFromRoot(treeOU) # all root to tip distances, and root to node distances
  names(dis) <- treeOU$tip.label
  D <- max(dis[1:n]) - dis[1:n]
  D <- D - mean(D)
  treeOU$edge.length[externalEdge] <- treeOU$edge.length[externalEdge] + D[des[externalEdge]]
}

# OPTIMISE the lik.BM and lik.OU functions:
opt1_2 <- optim.fun.repeated(3, lik.BM, my.init.fun.BM, tree = treeBM, VL = VL, mm = mm,
                             lower = my.lower.BM, upper = my.upper.BM)

opt2_2 <- optim.fun.repeated(3, lik.OU, my.init.fun.OU,
                             tree = treeOU, VL = VL, D = D, root.to.tip = root.to.tip, mm = mm, # D has to be passed on, and root to tip as well
                             lower = my.lower.OU, upper = my.upper.OU)

# 3. GET THE ML ESTIMATES OF THE ROOT AND OPTIMUM
par.white <- lm0$coefficients
par.BM <- lik.BM(par = opt1_2$pars, tree = treeBM, VL = VL, mm = mm, full.output = T)$beta
par.OU <- lik.OU(par = opt2_2$pars, tree = treeOU, VL = VL, D = D, root.to.tip = root.to.tip, full.output = T, mm = mm)$beta # the ML ancestral trait Y_A and theta in this order

# 4. COMPUTE HERITABILITY (WE RE-SIMULATE UNDER THE ML PARAMETERS AND IDENTIFY THE G AND E TERMS)
pat.dist.mat <- cophenetic.phylo(subtree)
vcv.mat <- vcv.phylo(subtree)

h2.BM <- get.h2(pars = opt1_2$pars, vars= list(vcv.mat), model = "BM", mm = mm, fixed.effects = par.BM, nsim = 1000, check.sim = T)
h2.OU <- get.h2(pars = opt2_2$pars, vars= list(vcv.mat, pat.dist.mat), model = "OU", mm = mm, fixed.effects = par.OU, nsim = 1000, check.sim = T)



